#include "calibrationrequest.hh"
#include "referencepoint.hh"



/* *** Constructors *** */


CalibrationRequest::
CalibrationRequest(uint_fast8_t _type = OWL_REQUEST_AUTOCALIBRATION):
  reference_point(NULL), direction(Direction())
{
  type = _type ;
}


CalibrationRequest::
CalibrationRequest(const Request &source,
                   ReferencePoint *_reference_point = NULL,
                   const Direction &_direction = Direction(),
                   uint_fast8_t _type = OWL_REQUEST_AUTOCALIBRATION):
  Request(source), reference_point(_reference_point),
  direction(_direction)
{
  type = _type ;
}



/* *** Write accessors *** */


void CalibrationRequest::reference_point_backward_link() const
{
  reference_point->add_request(this) ;
}


void CalibrationRequest::clear()
{
  Request::clear() ;
  reference_point = NULL ;
  direction.clear() ;
}



/* *** Operators *** */


const CalibrationRequest&
CalibrationRequest::operator=(const CalibrationRequest &source)
{
  if (this == &source)
    return *this ;

  this->Request::operator=(source) ;
  direction = source.direction ;
  reference_point = source.reference_point ;

  return *this ;
}


bool CalibrationRequest::operator==(const CalibrationRequest &source) const
{
  if (this == &source)
    return true ;

  return
    this->Request::operator==(source) &&
    direction == source.direction &&
    reference_point == source.reference_point ;
}



size_t hash_value(const CalibrationRequest &source)
{
  size_t seed = 0 ;

  boost::hash_combine(seed, static_cast<Request>(source)) ;
  boost::hash_combine(seed, source.direction) ;
  if (source.reference_point != NULL)
    boost::hash_combine(seed, *source.reference_point) ;

  return seed ;
}
