#ifndef _OWLPS_POSITIONING_USERINTERFACE_HH_
#define _OWLPS_POSITIONING_USERINTERFACE_HH_

#include <string>
#include <boost/program_options/options_description.hpp>

/// Handles configuration inputs from user
class UserInterface
{
protected:
  int cli_argument_count ;
  char **cli_argument_values ;
  boost::program_options::options_description *cli_options ;
  boost::program_options::options_description *file_options ;

  /** @name Operations */
  //@{
  /// Fills in all the options
  void fill_options(void) ;

  /// Fills in the options accepted on the command line
  void fill_cli_options(void) ;

  /// \brief Fills in the options accepted on the command line and in
  /// the configuration file
  void fill_file_options(void) ;
  void fill_data_input_options(void) ;
  void fill_input_options(void) ;
  void fill_log_options(void) ;
  void fill_positioning_options(void) ;
  void fill_output_options(void) ;
  void fill_misc_options(void) ;

  /// Parses all the configuration inputs and updates Configuration
  void parse_options(void) ;
  void parse_command_line(void) const ;
  void parse_file(void) const ;

  /// Prints usage and/or version information if requested by the user
  void print_information_and_exit_if_requested(void) const ;
  //@}

public:
  UserInterface(const int argc, char **argv) ;

  ~UserInterface(void) ;
} ;

#endif // _OWLPS_POSITIONING_USERINTERFACE_HH_
