#include "waypoint.hh"
#include "building.hh"

#include <sstream>

using namespace std ;
using std::tr1::unordered_set ;



/* *** Constructors *** */


/**
 * @param _b A pointer to the (first) Building to add to #buildings.
 * If it is NULL, #buildings will remain empty.
 * @param _x X coordinate.
 * @param _y Y coordinate.
 * @param _z Z coordinate.
 */
Waypoint::Waypoint(const Building *_b,
                   const float &_x, const float &_y, const float &_z
                  ): Point3D(_x, _y, _z)
{
  if (_b != NULL)
    buildings.insert(const_cast<Building*>(_b)) ;
}


/**
 * @param _b A pointer to the (first) Building to add to #buildings.
 * If it is NULL, #buildings will remain empty.
 * @param p Coordinates of the Waypoint.
 */
Waypoint::Waypoint(const Building *_b, const Point3D &p): Point3D(p)
{
  if (_b != NULL)
    buildings.insert(const_cast<Building*>(_b)) ;
}


/**
 * Clears #buildings, but does not deallocate the values pointed by
 * the elements into it.
 */
Waypoint::~Waypoint()
{
  buildings.clear() ;
}



/* *** Operators *** */


const Waypoint& Waypoint::operator=(const Waypoint &wp)
{
  if (this == &wp)
    return *this ;

  this->Point3D::operator=(wp) ;
  buildings = wp.buildings ;

  return *this ;
}


bool Waypoint::operator==(const Waypoint &wp) const
{
  if (this == &wp)
    return true ;

  return
    this->Point3D::operator==(wp) &&
    buildings == wp.buildings ;
}


Waypoint::operator std::string() const
{
  ostringstream csv ;
  csv << (Point3D) *this ;

  for (unordered_set<Building*>::const_iterator i = buildings.begin() ;
       i != buildings.end() ; ++i)
    csv << ';' << (*i)->get_name() ;

  return csv.str() ;
}



ostream &operator<<(ostream &os, const Waypoint &wp)
{
  // Coordinates
  os << (Point3D) wp ;

  // List of buildings
  if (wp.buildings.empty())
    os << '\n' << "Belongs to no building!" ;
  else
    for (unordered_set<Building*>::const_iterator i =
           wp.buildings.begin() ; i != wp.buildings.end() ; ++i)
      os << '\n' << **i ;

  return os ;
}
