#include <cxxtest/TestSuite.h>

#include "accesspoint.hh"

class AccessPoint_test: public CxxTest::TestSuite
{
public:

  void test_constructors(void)
  {
    // Default constructor
    AccessPoint accesspoint1 ;
    AccessPoint accesspoint2(Point3D(), "", "",
                             AP_DEFAULT_ANTENNA_GAIN,
                             WIFIDEVICE_DEFAULT_TRX_POWER,
                             AP_DEFAULT_CHANNEL) ;
    TS_ASSERT_EQUALS(accesspoint1, accesspoint2) ;

    // Copy constructor
    AccessPoint accesspoint3(Point3D(4,3,5), "192.168.0.1",
                             "aa:bb:cc:dd:ee:ff", 5, 32, 11) ;
    AccessPoint accesspoint4(accesspoint3) ;
    TS_ASSERT_EQUALS(accesspoint3, accesspoint4) ;
  }


  void test_accessors(void)
  {
    // Simple read accessors
    Point3D point3d1(78,23,4) ;
    AccessPoint accesspoint1(point3d1, "192.168.0.1",
                             "aa:bb:cc:dd:ee:ff", 6, 38, 10) ;
    TS_ASSERT_EQUALS(accesspoint1.get_coordinates(), point3d1) ;
    TS_ASSERT_EQUALS(accesspoint1.get_ip_addr(), "192.168.0.1") ;
    TS_ASSERT_EQUALS(accesspoint1.get_mac_addr(), "aa:bb:cc:dd:ee:ff") ;
    TS_ASSERT_EQUALS(accesspoint1.get_antenna_gain(), 6) ;
    TS_ASSERT_EQUALS(accesspoint1.get_trx_power(), 38) ;
    TS_ASSERT_EQUALS(accesspoint1.get_frequency(),
                     PosUtil::wifi_channel_to_hz(10)) ;

    // Write & read accessors
    Point3D point3d2(78,23,4) ;
    accesspoint1.set_coordinates(point3d2) ;
    TS_ASSERT_EQUALS(accesspoint1.get_coordinates(), point3d2) ;
    accesspoint1.set_ip_addr("10.0.8.42") ;
    TS_ASSERT_EQUALS(accesspoint1.get_ip_addr(), "10.0.8.42") ;
    accesspoint1.set_mac_addr("00:11:22:33:44:55") ;
    TS_ASSERT_EQUALS(accesspoint1.get_mac_addr(), "00:11:22:33:44:55") ;
    accesspoint1.set_antenna_gain(12.4) ;
    TS_ASSERT_DELTA(accesspoint1.get_antenna_gain(), 12.4, 0.00001) ;
    accesspoint1.set_trx_power(22.7) ;
    TS_ASSERT_DELTA(accesspoint1.get_trx_power(), 22.7, 0.00001) ;
    accesspoint1.set_channel(13) ;
    TS_ASSERT_EQUALS(accesspoint1.get_frequency(),
                     PosUtil::wifi_channel_to_hz(13)) ;
    accesspoint1.set_frequency(2423) ;
    TS_ASSERT_EQUALS(accesspoint1.get_frequency(), 2423u) ;
  }


  void test_operators(void)
  {
    // ==
    Point3D point3d1(78,23,4) ;
    AccessPoint accesspoint1(point3d1, "192.168.0.1",
                             "aa:bb:cc:dd:ee:ff", 6, 38, 10) ;
    AccessPoint accesspoint2(point3d1, "192.168.0.1",
                             "aa:bb:cc:dd:ee:ff", 6, 38, 10) ;
    TS_ASSERT_EQUALS(accesspoint1, accesspoint2) ;

    // !=
    Point3D point3d2(7,3,24) ;
    AccessPoint accesspoint3(point3d1, "10.0.0.1",
                             "aa:bb:cc:dd:ee:ff", 6, 38, 10) ;
    TS_ASSERT(accesspoint2 != accesspoint3) ;

    // =
    accesspoint2 = accesspoint3 ;
    TS_ASSERT_EQUALS(accesspoint2, accesspoint3) ;
  }

} ;
